@@ -8,6 +8,7 @@ from astrape.models.models_lightning import *
 import pytorch_lightning as pl
 from pytorch_lightning import loggers as pl_loggers
 from sklearn.model_selection import  train_test_split
+from sklearn.metrics import roc_auc_score
 import numpy as np
 from typing import Union, List, Dict, Any, Optional, cast, overload
 
@@ -22,7 +23,6 @@ import warnings
 warnings.filterwarnings(action='ignore')
 import shutil
 from pytorch_lightning.callbacks import RichProgressBar, TQDMProgressBar
-import random
 
 class BaseExperiment:
     """
@@ -423,16 +423,25 @@ class BaseExperiment:
         log_dir = folder_dir + "/" + self.model_metadata + "/" + "version_{}".format(version)
         self.logger = SummaryWriter(log_dir=log_dir)
         self.model.fit(self.X_train, self.y_train, sample_weight)
+
         y_train_preds = self.model.predict(self.X_train)
         train_acc = (self.y_train==y_train_preds).astype(float).mean()
         y_val_preds = self.model.predict(self.X_val)
         val_acc = (self.y_val==y_val_preds).astype(float).mean()
         y_test_preds = self.model.predict(self.X_test)
         test_acc = (self.y_test==y_test_preds).astype(float).mean()
-        
+        y_train_predict_proba = self.model.predict_proba(self.X_train)
+        train_auc = roc_auc_score(y_true=self.y_train, y_score=y_train_predict_proba)
+        y_val_predict_proba = self.model.predict_proba(self.X_val)
+        val_auc = roc_auc_score(y_true=self.y_val, y_score=y_val_predict_proba)
+        y_test_predict_proba = self.model.predict_proba(self.X_test)
+        test_auc = roc_auc_score(y_true=self.y_test, y_score=y_test_predict_proba)
         self.logger.add_scalar('val/acc', val_acc)
         self.logger.add_scalar('train/acc', train_acc)
         self.logger.add_scalar('test/acc', test_acc)
+        self.logger.add_scalar('val/auc', val_auc)
+        self.logger.add_scalar('train/auc', train_auc)
+        self.logger.add_scalar('test/auc', test_auc)
         if sample_weight:
             self.logger.add_scalar('sample_weight', sample_weight)
         self.logger.flush()
@@ -560,9 +569,10 @@ class BaseExperiment:
         
         has_val_loss = False
         has_val_acc = False
+        has_val_auc = False
+
         if isinstance(self.model, pl.LightningModule):
             logdir = self.logger.experiment.get_logdir()
-            
         elif isinstance(self.model, BaseEstimator):
             logdir = self.logger.get_logdir()
             
@@ -577,6 +587,15 @@ class BaseExperiment:
             has_val_acc = True
         except:
             pass
+        try:
+            val_auc_df = pd.DataFrame(
+                [(w, s, t) for w, s, t in event.Scalars('val/auc')],
+             columns=['wall_time', 'step', 'val/auc']
+            )
+            val_auc = val_auc_df['val/auc'][val_auc_df.index[-1]]
+            has_val_auc = True
+        except:
+            pass 
         try:    
             val_loss_df = pd.DataFrame(
                 [(w, s, t) for w, s, t in event.Scalars('val/loss')],
@@ -588,20 +607,21 @@ class BaseExperiment:
         except:
             pass
 
-        if not has_val_loss and not has_val_acc:
+        if not has_val_loss and not has_val_acc and not has_val_auc:
             raise AssertionError("Did not log anything.")
 
         val_loss = math.inf if not has_val_loss else val_loss
-        val_acc = 0 if not has_val_acc else val_acc
+        val_acc = -math.inf if not has_val_acc else val_acc
+        val_auc = -math.inf if not has_val_auc else val_auc
 
         if isinstance(self.model, BaseEstimator):
-            self.stack.append({"model" : self.model, "val_metrics": {'val/acc' : val_acc, 'val/loss' : val_loss}})
+            self.stack.append({"model" : self.model, "val_metrics": {'val/acc' : val_acc, 'val/loss' : val_loss, 'val/auc' : val_auc}})
             self.update_exp_metadata()   
         elif isinstance(self.model, pl.LightningModule):
-            self.stack.append({"model" : self.model, "trainer" : self.trainer, "val_metrics": {'val/acc' : val_acc, 'val/loss' : val_loss}})
+            self.stack.append({"model" : self.model, "trainer" : self.trainer, "val_metrics": {'val/acc' : val_acc, 'val/loss' : val_loss, 'val/auc' : val_auc}})
             self.update_exp_metadata()   
         else:
-            raise NotImplementedError("Expytorch-Lightning v 0.0.0 only supports variants of sklearn and pl.LightningModule.")
+            raise NotImplementedError("Astrape only supports variants of sklearn and pl.LightningModule.")
    
     #######################################################################  save_to_stack (Ends)  #######################################################################
 
@@ -666,7 +686,7 @@ class BaseExperiment:
         except:
             print("Nothing has been saved yet.")
             return None
-        mode = "max" if val_metric == "val/acc" else "min"
+        mode = "min" if val_metric == "val/loss" else "max"
         best_val = None
         ckpt_extension_type = None
         model_type = None
@@ -743,8 +763,8 @@ class BaseExperiment:
         self, 
         val_metric : str = "val/acc"
     ):
-        if val_metric not in ["val/loss", "val/acc"]:
-            raise ValueError("Wrong type of val_metric. It should be one of [{}]".format(*["val/loss", "val/acc"]))
+        if val_metric not in ["val/loss", "val/acc", "val/auc"]:
+            raise ValueError("Wrong type of val_metric. It should be one of [{}]".format(*["val/loss", "val/acc", "val/auc"]))
         if len(self.stack) == 0:
             print("Nothing is stored in stack.")
             return None
@@ -761,6 +781,9 @@ class BaseExperiment:
             elif (val_metric == "val/acc" and val_score > best_val_score):
                 best_val_score = val_score
                 best_idx = idx
+            elif (val_metric == "val/auc" and val_score > best_val_score):
+                best_val_score = val_score
+                best_idx = idx 
         
         best_ckpt = self.stack[best_idx]
 
@@ -852,23 +875,32 @@ class BaseExperiment:
                 val_acc = event.Scalars('val/acc')
                 log.update({'val/acc' : val_acc})
             except KeyError:
-                    error_count += 1 
+                error_count += 1 
             try:    
                 val_loss = event.Scalars('val/loss')
                 log.update({'val/loss' : val_loss})
             except KeyError:
-                    error_count += 1 
+                error_count += 1 
+            try:
+                val_auc = event.Scalars('val/auc')
+                log.update({'val/auc' : val_auc})
+            except KeyError:
+                error_count += 1
             try:
                 train_acc = event.Scalars('train/acc')
                 log.update({'train/acc' : train_acc})
             except KeyError:
-                    error_count += 1 
+                error_count += 1 
             try:    
                 train_loss = event.Scalars('train/loss')
                 log.update({'train/loss' : train_loss})
             except KeyError:
-                    error_count += 1 
-
+                error_count += 1 
+            try:
+                train_auc = event.Scalars('train/auc')
+                log.update({'train/auc' : train_auc})
+            except KeyError:
+                error_count += 1
         else: # save from input (metrics_dict)
             try:
                 log.update({'val/acc' : metrics_dict['val/acc']})
@@ -886,7 +918,15 @@ class BaseExperiment:
                 log.update({'train/loss' : metrics_dict['train/loss']})
             except KeyError:
                 error_count += 1
-        if error_count == 4:
+            try:
+                log.update({'val/auc' : metrics_dict['val/auc']})
+            except KeyError:
+                error_count += 1
+            try:
+                log.update({'train/auc' : metrics_dict['train/auc']})
+            except KeyError:
+                error_count += 1
+        if error_count == 6:
             raise AssertionError("Nothing has been logged.")
         
         with open(json_logdir, 'w') as fout:
